package Main

import Pokemon.*
import Scene.*
import SFML.Graphics.RenderWindow
import SFML.Window.{Event, Mouse}

class Game(window: RenderWindow, private var _currentScene: Scene):
    var isTrainerTurn = true

    val trainer = Trainer("", List(PokemonFactory("Bulbasaur"), PokemonFactory("Squirtle")), 0)
    private var trainerPokemonIndex = 1

    val trainerEnemy = Trainer("", List(PokemonFactory("Charmander")), 0)
    private var trainerPokemonEnemyIndex = 1

    var mouse = (0, 0)
    var mouseButtonClicked = false
    var queryScene: Option[Scene] = Some(_currentScene)

    def currentScene: Scene = _currentScene

    def fetchEvents() =
        if !isTrainerTurn then
            val pokemonTrainerEnemy = trainerEnemy.primaryPokemon.get
            val capacity =
                pokemonTrainerEnemy.capacity(scala.util.Random.between(0, pokemonTrainerEnemy.capacity.length))
            pokemonTrainerEnemy.useCapacity(capacity, trainer.primaryPokemon.get)
            this.isTrainerTurn = true

        if !trainer.primaryPokemon.get.isAlive then
            if trainerPokemonIndex == trainer.numberPokemon then
                window.close()
                println("Tous tes Pokémons sont hors combat. Fin de partie.")
                scala.sys.exit()
            trainer.changePrimaryPokemon(trainerPokemonIndex)
            trainerPokemonIndex = trainerPokemonIndex + 1

        if !trainerEnemy.primaryPokemon.get.isAlive then
            if trainerPokemonEnemyIndex == trainerEnemy.numberPokemon then
                window.close()
                println("Tous les Pokémons adverses sont hors combat. Fin de partie.")
                scala.sys.exit()
            trainerEnemy.changePrimaryPokemon(trainerPokemonEnemyIndex)
            trainerPokemonEnemyIndex = trainerPokemonEnemyIndex + 1

        if queryScene.isDefined then
            _currentScene.components = List()
            _currentScene = queryScene.get
            _currentScene.load(this)
            queryScene = None

        mouseButtonClicked = false
        for event <- window.pollEvent() do
            event match
                case Event.Closed()                                    => window.close()
                case Event.MouseMoved(x, y)                            => mouse = (x, y)
                case Event.MouseButtonPressed(Mouse.Button.Left, _, _) => mouseButtonClicked = true
                case _                                                 => ()
