package Engine.Objects

import scala.collection.mutable.ListBuffer
import scala.quoted.*
import scala.reflect.*

import Engine.Components.Component

import SFML.System.Vector2f

class GameObject private ():
    private var _childs: ListBuffer[GameObject] = ListBuffer()
    private var _components: ListBuffer[Component] = ListBuffer()
    private var _parent: Option[GameObject] = None
    private var _pos: Vector2f = Vector2f(0, 0)
    private var _scale: Float = 1.0f

    def this(pos: Vector2f = Vector2f(0, 0), scale: Float = 1.0f) =
        this()
        _pos = pos
        _scale = scale

    def addChild[T <: GameObject](child: T): T =
        child._parent = Some(this)
        _childs += child
        return child

    def recursiveChilds: ListBuffer[GameObject] =
        val recursiveChilds: ListBuffer[GameObject] = ListBuffer()

        for child <- _childs do recursiveChilds ++= (child +: child.recursiveChilds)

        return recursiveChilds

    def moveTo(pos: Vector2f): this.type = { _pos = pos; this }

    def pos: Vector2f = _parent match
        case Some(parent) => _pos * scale + parent.pos
        case None         => _pos

    def scale: Float = _parent match
        case Some(parent) => _scale * parent.scale
        case None         => _scale

    inline def addComponent[T <: Component](inline varargs: Any*): T =
        ${ GameObject.constructComponent[T]('this, 'varargs) }

    def getComponents[T <: Component]()(using TypeTest[Component, T]): ListBuffer[T] =
        return _components
            .filter(_ match
                case x: T => true
                case _    => false
            )
            .asInstanceOf[ListBuffer[T]]

object GameObject:
    def constructComponent[T <: Component: Type](gameObject: Expr[GameObject], varargs: Expr[Seq[Any]])(using
        Quotes
    ): Expr[T] =
        import quotes.reflect.*

        val Varargs(args) = varargs
        val argsTerm = args.map(_.asTerm).toList

        val componentASTConstructor =
            Apply(Select(New(TypeTree.of[T]), TypeRepr.of[T].typeSymbol.primaryConstructor), argsTerm).asExprOf[T]
        '{
            val component = $componentASTConstructor;
            component.gameObject = $gameObject;
            $gameObject._components += component;
            component
        }
