package Scene.Battle

import Component.*
import Scene.*

import SFML.Graphics.{Font, IntRect, Texture}

object BattleScene extends Scene:
    private def healthPercentage(life: Int, maxLife: Int): Int = (48 * (life.toFloat / maxLife.toFloat)).toInt

    def load(game: Main.Game) =
        val pokemonTrainer = game.trainer.primaryPokemon.get
        val pokemonTrainerEnemy = game.trainerEnemy.primaryPokemon.get

        val background = Sprite(Texture("src/main/resources/battle/background.png"))
        background.scale = 4

        val tiles = Sprite(Texture("src/main/resources/battle/tiles.png"))
        tiles.scale = 4

        val bar = Sprite(Texture("src/main/resources/battle/bar.png"))
        bar.scale = 4
        bar.pos = (544, 390)

        val barEnemy = Sprite(Texture("src/main/resources/battle/bar_enemy.png"))
        barEnemy.scale = 4
        barEnemy.pos = (0, 90)

        val healthBar = Sprite(Texture("src/main/resources/battle/health_bar_green.png"))
        healthBar.scale = 4
        healthBar.pos = (792, 466)
        healthBar.rect = Some(healthPercentage(pokemonTrainer.life, pokemonTrainer.maxLife), 7)

        val healthBarEnemy = Sprite(Texture("src/main/resources/battle/health_bar_green.png"))
        healthBarEnemy.scale = 4
        healthBarEnemy.pos = (200, 166)
        healthBarEnemy.rect = Some(healthPercentage(pokemonTrainerEnemy.life, pokemonTrainerEnemy.maxLife), 7)

        val textbox = Sprite(Texture("src/main/resources/textbox.png"))
        textbox.scale = 4
        textbox.pos = (0, 576)

        val pokemon = Sprite(Texture(f"src/main/resources/pokemons/back/${pokemonTrainer.id}%03d.png"))
        pokemon.scale = 4
        pokemon.pos = (130, 320)

        val pokemonEnemy = Sprite(Texture(f"src/main/resources/pokemons/front/${pokemonTrainerEnemy.id}%03d.png"))
        pokemonEnemy.scale = 4
        pokemonEnemy.pos = (645, 150)

        val fightBox = FightBox(Texture("src/main/resources/battle/fight_box.png"))
        fightBox.scale = 3
        fightBox.pos = (690, 603)

        val font = Font("src/main/resources/font.ttf")

        val name = Text(font)
        name.msg = pokemonTrainer.name
        name.color = 0.toByte
        name.pos = (624, 415)

        val nameEnemy = Text(font)
        nameEnemy.msg = pokemonTrainerEnemy.name
        nameEnemy.color = 0.toByte
        nameEnemy.pos = (35, 115)

        val health = Text(font)
        health.msg = s"${pokemonTrainer.life}"
        health.color = 0.toByte
        health.pos = (800, 495)

        val maxHealth = Text(font)
        maxHealth.msg = s"${pokemonTrainer.maxLife}"
        maxHealth.color = 0.toByte
        maxHealth.pos = (920, 495)

        val fight = Text(font)
        fight.msg = "Fight"
        fight.color = 255.toByte
        fight.pos = (745, 660)

        this.addComponent(background)
        this.addComponent(tiles)
        this.addComponent(bar)
        this.addComponent(healthBar)
        this.addComponent(barEnemy)
        this.addComponent(healthBarEnemy)
        this.addComponent(textbox)
        this.addComponent(pokemon)
        this.addComponent(pokemonEnemy)
        this.addComponent(fightBox)
        this.addComponent(name)
        this.addComponent(nameEnemy)
        this.addComponent(health)
        this.addComponent(maxHealth)
        this.addComponent(fight)
