package SFML.Wrapper
package Graphics

import scalanative.unsafe.*
import scalanative.unsigned.UnsignedRichInt

class RenderWindow(val mode: Window.VideoMode, val title: String, val style: Window.Window.WindowStyle) extends SFMLBind[SFML.Bindings.Graphics.RenderWindow.sfRenderWindow]:
    import SFML.Bindings.Graphics.RenderWindow.*

    private var renderWindow: Ptr[sfRenderWindow] =
        Zone { implicit z => sfRenderWindow_create(mode.bind(z), toCString(title), style.value.toUInt, null) }

    def bind(implicit z: Zone): Ptr[sfRenderWindow] = renderWindow

    def clear(color: Color): Unit =
        Zone { implicit z => sfRenderWindow_clear(renderWindow, color.bind(z)) }

    def close(): Unit =
        sfRenderWindow_close(renderWindow)

    def display(): Unit =
        sfRenderWindow_display(renderWindow)

    def draw(sprite: Sprite): Unit =
        Zone { implicit z => sfRenderWindow_drawSprite(renderWindow, sprite.bind(z), null) }

    def draw(text: Text): Unit =
        Zone { implicit z => sfRenderWindow_drawText(renderWindow, text.bind(z), null) }

    def isOpen(): Boolean =
        sfRenderWindow_isOpen(renderWindow) == SFML.Bindings.Type.sfTrue

    def pollEvent(): LazyList[Window.Event] =
        import SFML.Bindings.Window.Event.sfEvent

        def polling(event: Ptr[sfEvent]): Window.Event =
            if sfRenderWindow_pollEvent(renderWindow, event) == SFML.Bindings.Type.sfTrue then {
                return Window.FactoryEvent(event).getOrElse(polling(event))
            }
            return null

        Zone { implicit z =>
            val event = alloc[sfEvent]()

            return LazyList.continually(polling(event)).takeWhile(_ != null)
        }

    def setFramerateLimit(limit: Int): Unit =
        sfRenderWindow_setFramerateLimit(renderWindow, limit.toUInt)

    def setVerticalSyncEnabled(enabled: Boolean): Unit =
        val sfEnabled = if enabled then SFML.Bindings.Type.sfTrue else SFML.Bindings.Type.sfFalse
        sfRenderWindow_setVerticalSyncEnabled(renderWindow, sfEnabled)
